use cgmath::{Deg, EuclideanSpace, Matrix2, Point2, Vector2};
use rand::Rng;

use super::super::game::Projectile;
use super::ShipOutfit;
use super::{outfits::ShipOutfits, InputStatus, ShipControls, ShipTickResult};
use crate::{
	content,
	physics::PhysicsBody,
	render::{Sprite, SpriteTexture, Spriteable},
};

pub struct Ship {
	pub physicsbody: PhysicsBody,
	pub controls: ShipControls,
	outfits: ShipOutfits,

	sprite: SpriteTexture,
	size: f32,
}

impl Ship {
	pub fn new(ct: &content::Ship, outfits: Vec<ShipOutfit>, pos: Point2<f32>) -> Self {
		let mut o = ShipOutfits::new(ct.engines.clone(), ct.guns.clone());
		for x in outfits.into_iter() {
			o.add(x)
		}

		Ship {
			physicsbody: PhysicsBody::new(pos),
			controls: ShipControls::new(),

			outfits: o,
			sprite: SpriteTexture(ct.sprite.clone()),
			size: ct.size,
		}
	}

	pub fn update_controls(&mut self, input: &InputStatus) {
		self.controls.thrust = input.key_thrust;
		self.controls.right = input.key_right;
		self.controls.left = input.key_left;
		self.controls.guns = input.key_guns;
	}

	pub fn fire_guns(&mut self) -> Vec<Projectile> {
		let mut rng = rand::thread_rng();

		let mut out = Vec::new();

		for (g, p) in self.outfits.iter_guns_points() {
			if g.cooldown > 0.0 {
				continue;
			}

			g.cooldown = g.kind.rate + rng.gen_range(-g.kind.rate_rng..=g.kind.rate_rng);

			let pos = self.physicsbody.pos
				+ (Matrix2::from_angle(self.physicsbody.angle) * p.pos.to_vec());

			let vel = self.physicsbody.vel
				+ (Matrix2::from_angle(
					self.physicsbody.angle
						+ Deg(rng.gen_range(-(g.kind.spread.0 / 2.0)..=g.kind.spread.0 / 2.0)),
				) * Vector2 {
					x: 0.0,
					y: g.kind.projectile.speed
						+ rng.gen_range(-g.kind.projectile.speed_rng..=g.kind.projectile.speed_rng),
				});

			out.push(Projectile {
				position: pos,
				velocity: vel,
				angle: self.physicsbody.angle,
				sprite: SpriteTexture(g.kind.projectile.sprite.clone()),
				lifetime: g.kind.projectile.lifetime
					+ rng.gen_range(
						-g.kind.projectile.lifetime_rng..=g.kind.projectile.lifetime_rng,
					),
				size: g.kind.projectile.size
					+ rng.gen_range(-g.kind.projectile.size_rng..=g.kind.projectile.size_rng),
			})
		}
		return out;
	}

	pub fn tick(&mut self, t: f32) -> ShipTickResult {
		if self.controls.thrust {
			for e in self.outfits.iter_engines() {
				self.physicsbody.thrust(e.thrust * t);
			}
		}

		if self.controls.right {
			self.physicsbody.rot(Deg(35.0) * t);
		}

		if self.controls.left {
			self.physicsbody.rot(Deg(-35.0) * t);
		}

		let p = if self.controls.guns {
			self.fire_guns()
		} else {
			Vec::new()
		};

		self.physicsbody.tick(t);
		for i in self.outfits.iter_guns() {
			i.cooldown -= t;
		}

		return ShipTickResult { projectiles: p };
	}
}

impl Spriteable for Ship {
	fn get_sprite(&self) -> Sprite {
		Sprite {
			pos: (self.physicsbody.pos.x, self.physicsbody.pos.y, 1.0).into(),
			texture: self.sprite.clone(), // TODO: sprite texture should be easy to clone
			angle: self.physicsbody.angle,
			size: self.size,

			children: if self.controls.thrust {
				Some(self.outfits.get_engine_flares())
			} else {
				None
			},
		}
	}
}
